package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseDirectory;
import com.ruoyi.jjh.declaration.entity.BmsTemplateRecord;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordAddDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordUpdateDto;
import com.ruoyi.jjh.declaration.entity.vo.BmsTemplateRecordQueryVo;
import com.ruoyi.jjh.declaration.entity.vo.BmsTemplateRecordVo;
import com.ruoyi.jjh.declaration.mapper.BmsDeclarationRecordsMapper;
import com.ruoyi.jjh.declaration.mapper.BmsTemplateRecordMapper;
import com.ruoyi.jjh.declaration.regular.TimeChange;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseDirectoryService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.Year;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 在线模板Service业务层处理
 *
 * @author farben
 * @date 2023-09-07
 */
@Service
public class BmsTemplateRecordServiceImpl extends ServiceImpl<BmsTemplateRecordMapper, BmsTemplateRecord> implements IBmsTemplateRecordService {

    @Autowired
    private IBmsEnterpriseDirectoryService enterpriseDirectoryService;

    @Autowired
    private IBmsEnterpriseBasicInfoService iBmsEnterpriseBasicInfoService;
    private IBmsDeclarationRecordsService bmsDeclarationRecordsService;

    @Autowired
    private IBmsDeclarationRecordsService iBmsDeclarationRecordsService;
    @Resource
    private TimeChange tiTimeChange;

    /**
     * 查询在线模板
     *
     * @param id 在线模板主键
     * @return 在线模板
     */
    @Override
    public BmsTemplateRecordVo selectBmsTemplateRecordById(Long id) {
        BmsTemplateRecordVo bmsTemplateRecordVo = baseMapper.selectBmsTemplateRecordVoById(id);
        List<BmsEnterpriseDirectory> list = enterpriseDirectoryService.lambdaQuery()
                .eq(BmsEnterpriseDirectory::getTemplateRecordId, id).list();
        bmsTemplateRecordVo.setEnterpriseDirectoryList(list);
        return bmsTemplateRecordVo;
    }


    /**
     * 查询在线模板列表
     *
     * @param bmsTemplateRecordQueryDto 在线模板
     * @return 在线模板
     */
    @Override
    public List<BmsTemplateRecordQueryVo> selectBmsTemplateRecordList(BmsTemplateRecordQueryDto bmsTemplateRecordQueryDto) {
        return baseMapper.selectTemplateRecordVo(bmsTemplateRecordQueryDto);
    }

    @Override
    public Page<BmsTemplateRecordQueryVo> selectBmsTemplateRecordList(Page<BmsTemplateRecordQueryVo> page, BmsTemplateRecordQueryDto bmsTemplateRecordQueryDto) {
        return baseMapper.selectTemplateRecordVo(page, bmsTemplateRecordQueryDto);
    }

    @Override
    public List<BmsTemplateRecordQueryVo> selectTemplateRecordList(String creditCode) {
        return baseMapper.selectTemplateRecordList(creditCode);
    }

    @Override
    public List<BmsTemplateRecordQueryVo> selectBmsTemplateNameList(BmsTemplateQueryDto bmsTemplateQueryDto) {
        Long enterpriseId = bmsTemplateQueryDto.getEnterpriseId();
        BmsEnterpriseBasicInfo bmsEnterpriseBasicInfo = iBmsEnterpriseBasicInfoService.selectBmsEnterpriseBasicInfoById(enterpriseId);
        if (BeanUtil.isNotEmpty(bmsEnterpriseBasicInfo)) {
            bmsTemplateQueryDto.setCreditCode(bmsEnterpriseBasicInfo.getCreditCode());
            return baseMapper.selectBmsTemplateNameList(bmsTemplateQueryDto);
        }
        return null;
    }

    /**
     * 新增在线模板
     *
     * @param bmsTemplateRecordAddDto 在线模板
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertBmsTemplateRecord(BmsTemplateRecordAddDto bmsTemplateRecordAddDto) {
        int num = 0;
        QueryWrapper<BmsTemplateRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("enterprise_directory", bmsTemplateRecordAddDto.getEnterpriseDirectory());
        List<BmsTemplateRecord> isFlag = baseMapper.selectList(queryWrapper);
        if(!isFlag.isEmpty()){
            throw new ServiceException("该申报任务标题已经存在");
        }
        //如果不限时
        if(bmsTemplateRecordAddDto.getIsTime()==null){
            throw new ServiceException("请选择是否限时");
        } else if(bmsTemplateRecordAddDto.getIsTime() == 1){
            bmsTemplateRecordAddDto.setStatus(3);
        }else {
            //新增的时候根据当前时间判断状态是否有效
            tiTimeChange.getTime(bmsTemplateRecordAddDto);
        }
        //添加在线模板
        num = baseMapper.insert(bmsTemplateRecordAddDto);
        //前端excel企业名录集合
        List<BmsEnterpriseDirectory> enterpriseDirectoryList = bmsTemplateRecordAddDto
                .getEnterpriseDirectoryList();
        //要添加的在线申报记录
        List<BmsDeclarationRecords> drlist = new ArrayList<>();
        //如果前端部分企业名录集合为空
        if (null != enterpriseDirectoryList && enterpriseDirectoryList.size() > 0) {
            enterpriseDirectoryList.forEach(x -> {
                getDeclarationRecordsList(bmsTemplateRecordAddDto, drlist, x.getCreditCode(), bmsTemplateRecordAddDto.getTemplateId());
                x.setTemplateRecordId(bmsTemplateRecordAddDto.getId());
            });
            enterpriseDirectoryService.saveBatch(enterpriseDirectoryList);
        } else {
            List<BmsEnterpriseBasicInfo> list = iBmsEnterpriseBasicInfoService.list();
            List<BmsEnterpriseDirectory> copyList = new ArrayList<>();
            for (BmsEnterpriseBasicInfo items : list) {
                BmsEnterpriseDirectory bms = new BmsEnterpriseDirectory();
                bms.setTemplateRecordId(bmsTemplateRecordAddDto.getId());
                bms.setEnterpriseName(items.getEnterpriseName());
                bms.setCreditCode(items.getCreditCode());
                copyList.add(bms);
                getDeclarationRecordsList(bmsTemplateRecordAddDto, drlist, items.getCreditCode(), bmsTemplateRecordAddDto.getTemplateId());
            }
            enterpriseDirectoryService.saveBatch(copyList);
        }
        iBmsDeclarationRecordsService.saveBatch(drlist);
        return num;
    }

//    /**
//     * 假如给该企业已经加了申报任务指定申报模板，就不能再给这个企业加该申报模板的申报任务（所有的企业）
//     *
//     * @param list
//     */
//    private void judgement(List<BmsDeclarationRecords> bm, List<BmsEnterpriseDirectory> list) {
//        List<String> str1 = new ArrayList<>();
//        bm.forEach(x -> {
//            list.forEach(y -> {
//                if (Objects.equals(x.getEnterpriseId(), y.getId())) {
//                    str1.add(y.getEnterpriseName());
//                }
//            });
//        });
//        throw new ServiceException(StringUtils.join(str1, ",") + "已经分配该模板任务");
//    }

    private void getDeclarationRecordsList(BmsTemplateRecord bmsTemplateRecordAddDto, List<BmsDeclarationRecords> drlist, String creditCode, Long items) {
        BmsDeclarationRecords drItems = new BmsDeclarationRecords();
        drItems.setProjectName(bmsTemplateRecordAddDto.getEnterpriseDirectory());
        drItems.setYear(String.valueOf(Year.now().getValue()));
        drItems.setCreditCode(creditCode);
        drItems.setTemplateRecordId(bmsTemplateRecordAddDto.getId());
        drItems.setStatus(0L);
        drItems.setTemplateId(Math.toIntExact(items));
        drItems.setIsDeleted(0L);
        drlist.add(drItems);
    }

    /**
     * 修改在线模板
     *
     * @param bmsTemplateRecordUpdateDto 在线模板
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateBmsTemplateRecord(BmsTemplateRecordUpdateDto bmsTemplateRecordUpdateDto) {
        QueryWrapper<BmsTemplateRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("enterprise_directory", bmsTemplateRecordUpdateDto.getEnterpriseDirectory());
        List<BmsTemplateRecord> isFlag = baseMapper.selectList(queryWrapper);
        if(!isFlag.isEmpty()){
            throw new ServiceException("该申报任务标题已经存在");
        }
        tiTimeChange.getTime(bmsTemplateRecordUpdateDto);
        int  num =  baseMapper.updateById(bmsTemplateRecordUpdateDto);
        // 删除企业相关列表
        QueryWrapper<BmsEnterpriseDirectory> queryWrapper1 = new QueryWrapper<>();
        queryWrapper.eq("template_record_id", bmsTemplateRecordUpdateDto.getId());
        enterpriseDirectoryService.remove(queryWrapper1);
        QueryWrapper<BmsDeclarationRecords> queryWrapper2 = new QueryWrapper<>();
        queryWrapper.eq("template_record_id", bmsTemplateRecordUpdateDto.getId());
        bmsDeclarationRecordsService.remove(queryWrapper2);

        //前端excel企业名录集合
        List<BmsEnterpriseDirectory> enterpriseDirectoryList = bmsTemplateRecordUpdateDto
                .getEnterpriseDirectoryList();
        //要添加的在线申报记录
        List<BmsDeclarationRecords> drlist = new ArrayList<>();
        //如果前端部分企业名录集合为空
        if (null != enterpriseDirectoryList && enterpriseDirectoryList.size() > 0) {
            enterpriseDirectoryList.forEach(x -> {
                getDeclarationRecordsList(bmsTemplateRecordUpdateDto, drlist, x.getCreditCode(), bmsTemplateRecordUpdateDto.getTemplateId());
                x.setTemplateRecordId(bmsTemplateRecordUpdateDto.getId());
            });
            enterpriseDirectoryService.saveBatch(enterpriseDirectoryList);
        } else {
            List<BmsEnterpriseBasicInfo> list = iBmsEnterpriseBasicInfoService.list();
            List<BmsEnterpriseDirectory> copyList = new ArrayList<>();
            for (BmsEnterpriseBasicInfo items : list) {
                BmsEnterpriseDirectory bms = new BmsEnterpriseDirectory();
                bms.setTemplateRecordId(bmsTemplateRecordUpdateDto.getId());
                bms.setEnterpriseName(items.getEnterpriseName());
                bms.setCreditCode(items.getCreditCode());
                copyList.add(bms);
                getDeclarationRecordsList(bmsTemplateRecordUpdateDto, drlist, items.getCreditCode(), bmsTemplateRecordUpdateDto.getTemplateId());
            }
            enterpriseDirectoryService.saveBatch(copyList);
        }
        iBmsDeclarationRecordsService.saveBatch(drlist);
        return num;
    }

    /**
     * 批量删除在线模板
     *
     * @param ids 需要删除的在线模板主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateRecordByIds(Long[] ids) {
        return baseMapper.deleteBmsTemplateRecordByIds(ids);
    }

    /**
     * 删除在线模板信息
     *
     * @param id 在线模板主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateRecordById(Long id) {
        return baseMapper.deleteBmsTemplateRecordById(id);
    }

    /**
     * 补录
     *
     * @param bmsTemplateRecordUpdateDto
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int supplementation(BmsTemplateRecordUpdateDto bmsTemplateRecordUpdateDto) {
        BmsTemplateRecord infos = getById(bmsTemplateRecordUpdateDto.getId());
        if (infos.getIsTime() == 1){
            bmsTemplateRecordUpdateDto.setStartTime(null);
            bmsTemplateRecordUpdateDto.setEndTime(null);
            bmsTemplateRecordUpdateDto.setStatus(3);
        }else if(infos.getIsTime() == 0){
            //只有未修改之前是过期的才能补录
            if (bmsTemplateRecordUpdateDto.getStatus() != 1) {
                throw new ServiceException("未过期不可以补录");
            }
            tiTimeChange.getTime(bmsTemplateRecordUpdateDto);
            if (bmsTemplateRecordUpdateDto.getStatus() != 2) {
                throw new ServiceException("请修改时间范围为有效区间");
            }
        }else {
            throw new ServiceException("补录失败");
        }
        int num = baseMapper.updateById(bmsTemplateRecordUpdateDto);
        //要添加的在线申报记录
        List<BmsDeclarationRecords> drlist = new ArrayList<>();
        //获取前端传递的企业名录列表
        List<BmsEnterpriseDirectory> enterpriseDirectoryList = bmsTemplateRecordUpdateDto
                .getEnterpriseDirectoryList();
        //先去查找企业名录中对应该在线模板的相关企业
        List<BmsEnterpriseDirectory> list = enterpriseDirectoryService.lambdaQuery()
                .eq(BmsEnterpriseDirectory::getTemplateRecordId, bmsTemplateRecordUpdateDto.getId()).list();
        //首先部分企业和所有企业去区分
        if (bmsTemplateRecordUpdateDto.getIsFrame() == 0) {
            enterpriseDirectoryList = new ArrayList<>();
            //企业信息表里所有企业
            List<BmsEnterpriseBasicInfo> allList = iBmsEnterpriseBasicInfoService.list();
            //将信息表的所有企业添加到名录表
            for (BmsEnterpriseBasicInfo x : allList) {
                BmsEnterpriseDirectory bmsEnterpriseDirectory = new BmsEnterpriseDirectory();
                bmsEnterpriseDirectory.setCreditCode(x.getCreditCode());
                bmsEnterpriseDirectory.setEnterpriseName(x.getEnterpriseName());
                enterpriseDirectoryList.add(bmsEnterpriseDirectory);
            }
        }
        //java有两个list实体类集合 如果a集合某条数据里面的name和b里面某条数据的name相等 就去除a的该条数据
        enterpriseDirectoryList.removeIf(entityA -> list.stream().anyMatch(entityB -> entityB.getCreditCode().equals(entityA.getCreditCode())));
        enterpriseDirectoryList.forEach(x->{
            x.setTemplateRecordId(bmsTemplateRecordUpdateDto.getId());
            getDeclarationRecordsList(bmsTemplateRecordUpdateDto, drlist, x.getCreditCode(), bmsTemplateRecordUpdateDto.getTemplateId());
        });

        //新增到企业名录
        enterpriseDirectoryService.saveBatch(enterpriseDirectoryList);
        //最后新添加进在线申报记录
        iBmsDeclarationRecordsService.saveBatch(drlist);

        return num;
    }

    /**
     * 暂省略（和企业信息表做对比）
     *
     * @param enterpriseDirectoryList
     * @return
     */
    private List<BmsEnterpriseBasicInfo> filterEnterprise(List<BmsEnterpriseDirectory> enterpriseDirectoryList) {
        //保存所有的信用代码
        List<String> codeList = new ArrayList<>();
        for (BmsEnterpriseDirectory x : enterpriseDirectoryList) {
            codeList.add(x.getCreditCode());
        }
        //根据信用代码去企业信息表里查找该企业，如果能找到并且该企业未填报过该模板 就添加企业id
        LambdaQueryWrapper<BmsEnterpriseBasicInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(BmsEnterpriseBasicInfo::getCreditCode, codeList);
        return iBmsEnterpriseBasicInfoService.list(lambdaQueryWrapper);
    }
}
